// various utilities with no particular theme


unit QUB_Global;


interface


uses


Windows, SysUtils{, Dialogs, Forms},
Math, 
AllocTypes, Alloc;


const


ONE_MEGABYTE            = 1048576;

//MAX_CHANNEL             = 15;


DATA_MSG_RECOUNT_DATA   =    1;
DATA_MSG_BROWSE         =    2;
DATA_MSG_READ_CACHE     =    4;
DATA_MSG_DRAW_DATA      =    8;
DATA_MSG_PROC_DATA      =   16;
DATA_MSG_SET_SCROLL     =   32;
DATA_MSG_COND_BROWSE    =   64;
DATA_MSG_BROWSE_TO_P    =  128;
DATA_MSG_BROWSE_TO_T    =  256;
DATA_MSG_BROWSE_CMD     =  512;
DATA_MSG_DATA_EDIT      = 1024;
DATA_MSG_DATA_APPEND    = 2048;
DATA_MSG_SCROLLINVIEW   = 4096;
DATA_MSG_USE_ACC        = 8192;

DATA_MSG_NOT_BENIGN_FOR_ACC = not (DATA_MSG_BROWSE or
                                    DATA_MSG_DRAW_DATA or
                                    DATA_MSG_PROC_DATA);

type


TQUB_FirstLast          = record
  First                 :Integer;
  Last                  :Integer;
end;
TQUB_PFirstLast         = ^TQUB_FirstLast;

//TQUB_AFirstLast         = Array[0..(MaxInt div (SizeOf(TQUB_FirstLast)*2))] of TQUB_FirstLast;
//TQUB_PAFirstLast        = ^TQUB_AFirstLast;


function Choice(Value :Boolean; const A, B :String) :String; overload; forward;
function Choice(Value :Boolean; const A, B :Integer) :Integer; overload; forward;
function Choice(Value :Boolean; const A, B :Double) :Double; overload; forward;
//function Choice(Value :Boolean; const A, B :Variant) :Double; overload; forward;


function QUB_FirstLast(First, Last :Integer) :TQUB_FirstLast; forward;


function GetNewUniqueFileName(const BaseName, BaseExt :String; var NoNameCount :Integer) :String; forward;

function Gaussian(Mean, Std :Double; Value :Double) :Double; forward;
function SafeLog(Value :Double) :Double; forward;
function MatrixLogarithm(A, Q :TPMDouble; dt :Double; Size :Integer; MaxIter :Integer; Conv :Double) :Boolean; forward;

// Round to iDec decimals, iDec >= 0 {JB 02-2004}
function _round(n : Double; iDec:Integer): Double;

// test for nonnumeric, empty strings
function IsValidInteger( c :String ) : Boolean;

// Convert string to integer or return default if not a valid integer
function ValidStrToInt( c :String; iDefault : Integer ) : Integer;

implementation


//******************************************************************************
function Choice(Value :Boolean; const A, B :String) :String;
begin
  if Value then Result := A else Result := B;
end;
function Choice(Value :Boolean; const A, B :Integer) :Integer;
begin
  if Value then Result := A else Result := B;
end;
function Choice(Value :Boolean; const A, B :Double) :Double;
begin
  if Value then Result := A else Result := B;
end;
{function Choice(Value :Boolean; const A, B :Variant) :Double;
begin
  if Value then Result := A else Result := B;
end;
}

//******************************************************************************
function Gaussian(Mean, Std :Double; Value :Double) :Double; 
var
  tmp :Double;
begin
  Result := 0;
  try
    if Std = 0 then begin
      if Abs(Value - Mean) < 1e-100 then begin
        Result := 0;
      end else begin
        Result := 1e100//a substitute for infinity
      end;
    end else begin
      tmp := (Value - Mean) / Std;
      tmp := exp(-tmp * tmp / 2.0);
      tmp := tmp / (2.5066283 * Std);
      if tmp < 1e-100 then begin
        Result := 1e-100;
      end else begin
        Result := tmp;
      end;
    end;
  except
  end;
end;

function SafeLog(Value :Double) :Double;
begin
  Result := 0;
  try
    if Value <= 0 then begin
      Result := -1e100//a substitute for -infinity
    end else begin
      Result := ln(Value);
    end;
  except
  end;
end;


{
  Returns the logarithm of a matrix, defined as the (truncated) series:

    Sum(-[(I-A)^n]/n)
}
function MatrixLogarithm(A, Q :TPMDouble; dt :Double; Size :Integer; MaxIter :Integer; Conv :Double) :Boolean;
var
  IA, AN, QQ, QP, p   :TPMDouble;
  Converged           :Boolean;
  i, j, k, n          :Integer;
begin
  Result := False;

  IA := nil;
  AN := nil;
	QQ := nil;
  QP := nil;

  try


    try


      IA := DAlloc2(Size, Size);
      AN := DAlloc2(Size, Size);
      QQ := DAlloc2(Size, Size);
      QP := DAlloc2(Size, Size);


      //IA = I - A
      //AN = I
      //QQ = 0
      //QP = 0
      for i := 0 to Size - 1 do begin
        for j := 0 to Size - 1 do begin
          if i <> j then begin
          //off-diagonal
            IA[i, j] :=   - A[i, j];
            AN[i, j] := 0;
          end else begin
          //diagonal
            IA[i, j] := 1 - A[i, j];
            AN[i, j] := 1;
          end;
          QQ[i, j] := 0;
					QP[i, j] := 0;
        end;
      end;

      n := 1;

      repeat


        Converged := True;


        //AN := AN.IA;
        for i := 0 to Size - 1 do begin
          for j := 0 to Size - 1 do begin
            QQ[i, j] := 0;
            for k := 0 to Size - 1 do begin
              QQ[i, j] := QQ[i, j] + AN[i, k]*IA[k, j];
            end;
          end;
        end;
        for i := 0 to Size - 1 do for j := 0 to Size - 1 do AN[i, j] := QQ[i, j];

        //QQ := QP - AN/n;
        for i := 0 to Size - 1 do begin
          for j := 0 to Size - 1 do begin
            QQ[i, j] := QP[i, j] - AN[i, j];
          end;
        end;


				for i := 0 to Size - 1 do begin
          for j := 0 to Size - 1 do begin
            if Abs((QQ[i, j] + 1e-100 - QP[i, j])/(QQ[i, j] + 1e-100)) > Conv then begin
              Converged := False;
              Break;
            end;
          end;
          if not Converged then Break;
        end;

        p  := QQ;
        QQ := QP;
        QP := p;

        Inc(n);


      until((Converged) or (n > MaxIter));


      //Q := QQ / dt;
      for i := 0 to Size - 1 do begin
        for j := 0 to Size - 1 do begin
          Q[i, j] := QP[i, j] / dt;
        end;
      end


    except

      Result := True;

    end;


  finally

    Free2(TPMChar(IA));
    Free2(TPMChar(AN));
    Free2(TPMChar(QQ));
    Free2(TPMChar(QP));

  end;

end;


function GetNewUniqueFileName(const BaseName, BaseExt :String; var NoNameCount :Integer) :String;
begin
  Inc(NoNameCount);
  Result := BaseName + IntToStr(NoNameCount) + BaseExt;
  while FileExists(Result) do begin
    Inc(NoNameCount);
    Result := BaseName + IntToStr(NoNameCount) + BaseExt;
    if NoNameCount = 100000 then Break;
  end;
end;

{function UniqueFileName(const FileName, FileExt, Title, Filter :String) :String;
var
  Dlg            :TSaveDialog;
begin
	Result         := '';

  Dlg            := TSaveDialog.Create(nil);
  Dlg.Options    := [ofPathMustExist];
  Dlg.DefaultExt := FileExt;
  Dlg.Filter     := Filter;
  Dlg.Title      := Title;

  try

    Result := FileName;
    while(FileExists(ChangeFileExt(Result, FileExt))) do begin
      case MessageDlg('File ' + ChangeFileExt(Result, FileExt) + ' already exists. Do you want to ovewrite it ?', mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
        idYes:
          Break;
        idNo:
          begin
            Dlg.FileName := Result;
            if Dlg.Execute then begin
              Result := ChangeFileExt(Dlg.FileName, '');
            end;
          end;
        idCancel:
          begin
            Result := '';
            Exit;
          end;
      end;
    end;

  finally

    Dlg.Free;

  end;

end;
}


function QUB_FirstLast(First, Last :Integer) :TQUB_FirstLast;
begin
	Result.First := First;
	Result.Last  := Last;
end;

function _round(n : Double; iDec:Integer): Double;
	begin
		Result := Round( n  * IntPower( 10.0, iDec ) ) / IntPower( 10.0, iDec );
	end;
  
function IsValidInteger( c :String ) : Boolean;
  var
    i : Integer;
  begin
    result := True;
    try
      if Length(c)=0 Then result := False else i := StrToInt(c);
    except
      on EConvertError do result := False;
    end;
  end;

function ValidStrToInt( c :String; iDefault : Integer ) : Integer;
  begin
    if IsValidInteger( c ) Then Result := StrToInt(c) else Result := iDefault;
  end;

//******************************************************************************
end.
