#ifndef QUBTREE_NODE_H
#define QUBTREE_NODE_H

//
// Define API decoration for direct importing of DLL references.
//

#if defined(_WIN32)
#define DLLEXPORT __declspec(dllexport)
#define DLLIMPORT __declspec(dllimport)
#else
#define DLLEXPORT
#define DLLIMPORT
#endif

#if !defined(_QTR_PY_)
#define QTR_PY_API DLLIMPORT
#else
#define QTR_PY_API DLLEXPORT
#endif

#include <Python.h>
#include <QUB_Tree.h>

///

typedef struct {
  PyObject_HEAD
  QTR_Impl *impl;
} qubtree_node;

typedef struct {
  PyObject_HEAD
  QTR_Impl *impl;
} qubtree_data;

typedef struct {
  PyObject_HEAD
  QTR_Impl *impl;
  unsigned int row;
} qubtree_dataRow;

typedef struct {
  PyObject_HEAD
  QTR_Impl *impl;
  unsigned int col;
} qubtree_dataCol;

// c++ function
QTR_PY_API QUB_Tree qubtree_node_obj(PyObject *node); // works on qubtree_*

#ifdef __cplusplus    /*`__cplusplus' is #defined iff compiler is C++*/
extern "C" {
#endif

QTR_PY_API void init_qubtree();

QTR_PY_API PyObject* qubtree_node_new(const char *name);
QTR_PY_API PyObject* qubtree_node_using(QTR_Impl *impl);
QTR_PY_API int       qubtree_node_check(PyObject *node);
QTR_PY_API QTR_Impl* qubtree_node_impl(PyObject *node); // works on qubtree_*
void qubtree_node_dealloc(PyObject* self);

QTR_DataType qubtree_node_getSeqType(PyObject *seq);

PyObject* qubtree_node_outstanding(PyObject* self, PyObject* args);
PyObject* qubtree_node_outstandingRoots(PyObject* self, PyObject* args);

PyObject* qubtree_node_create(PyObject* self, PyObject* args);
PyObject* qubtree_node_createNull(PyObject* self, PyObject* args);
PyObject* qubtree_node_createComment(PyObject* self, PyObject* args);
PyObject* qubtree_node_open(PyObject* self, PyObject* args);
PyObject* qubtree_node_readText(PyObject* self, PyObject* args);
PyObject* qubtree_node_readString(PyObject* self, PyObject* args);
PyObject* qubtree_node_readBytes(PyObject* self, PyObject* args);

PyObject* qubtree_node_save(PyObject* self, PyObject* args);
PyObject* qubtree_node_saveAs(PyObject* self, PyObject* args);
PyObject* qubtree_node_saveAsText(PyObject* self, PyObject* args);
PyObject* qubtree_node_close(PyObject* self, PyObject* args);

PyObject *qubtree_node_repr(PyObject *self);
PyObject *qubtree_node_str(PyObject *self);
int qubtree_node_compare(PyObject* v, PyObject* w);

PyObject* qubtree_node_clone(PyObject* self, PyObject* args);
PyObject* qubtree_node_list(PyObject* self, PyObject* args);
PyObject* qubtree_node_find(PyObject* self, PyObject* args);
PyObject* qubtree_node_next(PyObject* self, PyObject* args);
PyObject* qubtree_node_nextSameName(PyObject* self, PyObject* args);
PyObject* qubtree_node_append(PyObject* self, PyObject* args);
PyObject* qubtree_node_appendClone(PyObject* self, PyObject* args);
PyObject* qubtree_node_insert(PyObject* self, PyObject* args);
PyObject* qubtree_node_insertClone(PyObject* self, PyObject* args);
PyObject* qubtree_node_remove(PyObject* self, PyObject* args);
// PyObject* qubtree_node_setChanged(PyObject* self, PyObject* args);

PyObject* qubtree_node_getBytes(PyObject* self, PyObject* args);

PyObject* qubtree_node_lock(PyObject* self, PyObject* args);
PyObject* qubtree_node_unlock(PyObject* self, PyObject* args);

int qubtree_node_length(PyObject *self);
PyObject *qubtree_node_subscript(PyObject *self, PyObject *key);

PyObject *qubtree_node_getattr(PyObject *self, char *name);
int qubtree_node_setattr(PyObject *self, char *name, PyObject *val);



PyObject *qubtree_data_new( QTR_Impl *impl );

void qubtree_data_dealloc(PyObject* self);
PyObject *qubtree_data_repr(PyObject *self);
PyObject *qubtree_data_str(PyObject *self);
int qubtree_data_length(PyObject *self);
PyObject *qubtree_data_getattr(PyObject *self, char *name);
int qubtree_data_setattr(PyObject *self, char *name, PyObject *val);

PyObject* qubtree_data_item( qubtree_data *ndata, int ix );
PyObject* qubtree_data_slice( qubtree_data *ndata, int from, int until );
int qubtree_data_ass_item(qubtree_data *ndata, int ix, PyObject *val);
int qubtree_data_ass_slice(qubtree_data *ndata, int from, int until, PyObject *val);

PyObject* qubtree_data_setup(PyObject *self, PyObject *args);
PyObject* qubtree_data_resize(PyObject *self, PyObject *args);
PyObject* qubtree_data_clear(PyObject *self, PyObject *args);
PyObject* qubtree_data_load(PyObject *self, PyObject *args);
PyObject* qubtree_data_unload(PyObject *self, PyObject *args);

PyObject* qubtree_data_row(PyObject *self, PyObject *args);
PyObject* qubtree_data_col(PyObject *self, PyObject *args);



PyObject *qubtree_dataRow_new( QTR_Impl *impl, int ix );

void qubtree_dataRow_dealloc(PyObject* self);
PyObject *qubtree_dataRow_repr(PyObject *self);
PyObject *qubtree_dataRow_str(PyObject *self);
int qubtree_dataRow_length(PyObject *self);

PyObject* qubtree_dataRow_item( PyObject *row, int ix );
PyObject* qubtree_dataRow_slice( PyObject *row, int from, int until );
int qubtree_dataRow_ass_item(PyObject *row, int ix, PyObject *val);
int qubtree_dataRow_ass_slice(PyObject *row, int from, int until, PyObject *val);



PyObject *qubtree_dataCol_new( QTR_Impl *impl, int ix );

void qubtree_dataCol_dealloc(PyObject* self);
PyObject *qubtree_dataCol_repr(PyObject *self);
PyObject *qubtree_dataCol_str(PyObject *self);
int qubtree_dataCol_length(PyObject *self);

PyObject* qubtree_dataCol_item( PyObject *col, int ix );
PyObject* qubtree_dataCol_slice( PyObject *col, int from, int until );
int qubtree_dataCol_ass_item(PyObject *col, int ix, PyObject *val);
int qubtree_dataCol_ass_slice(PyObject *col, int from, int until, PyObject *val);



#ifdef __cplusplus    /*`__cplusplus' is #defined iff compiler is C++*/
}
#endif



#endif
